from lastipy.track import Track

class PlaylistTrack(Track):
    """Represents a Spotify track that exists in a playlist"""

    def __init__(self, track_name, artist, spotify_id, added_at):
        self.track_name = track_name
        self.artist = artist
        self.spotify_id = spotify_id
        self.added_at = added_at

    def __eq__(self, other):
        return isinstance(other, Track) \
            and self.track_name == other.track_name \
            and self.artist == other.artist \
            and self.spotify_id == other.spotify_id \
            and self.added_at == other.added_at

    def __repr__(self):
        return str(self.__dict__)


import unittest
from unittest.mock import patch
from lastipy.lastfm.library import recent_artists
from unittest.mock import Mock
from requests import HTTPError
from lastipy.lastfm.library.scrobbled_artist import ScrobbledArtist


class RecentArtistsTest(unittest.TestCase):

    dummy_user = 'dummyUser'
    dummy_api_key = '123456789'

    @patch('lastipy.lastfm.library.recent_artists.fetch_paginated_response')
    def test_fetch(self, mock_paginated_endpoint_fetch):
        expected_artists = [
            ScrobbledArtist(artist_name='The Beatles', playcount=10),
            ScrobbledArtist(artist_name='Cream', playcount=4),
            ScrobbledArtist(artist_name='Led Zeppelin', playcount=7)
        ]

        mock_paginated_endpoint_fetch.return_value = [
            {
                'artists': {
                    'artist': [
                        {
                            'name': 'The Beatles',
                            'playcount': '10'
                        },
                        {
                            'name': 'Cream',
                            'playcount': '4'
                        }
                    ],
                    '@attr': {
                        'totalPages': '2'
                    }
                }
            },
            {
                'artists': {
                    'artist': [
                        {
                            'name': 'Led Zeppelin',
                            'playcount': '7'
                        }
                    ],
                    '@attr': {
                        'totalPages': '2'
                    }
                }
            }
        ]

        fetched_artists = recent_artists.fetch_recent_artists(
            user=self.dummy_user, api_key=self.dummy_api_key)

        self.assertCountEqual(fetched_artists, expected_artists)



class SpotifyAlbum():
    """Represents a Spotify "album". This is slightly confusing in that there are different "types" of albums: albums, and singles."""

    def __init__(self, album_type, spotify_id, release_date_precision, release_date):
        self.album_type = album_type
        self.spotify_id = spotify_id
        self.release_date_precision = release_date_precision
        self.release_date = release_date
    
    def __eq__(self, other):
        return isinstance(other, SpotifyAlbum) \
            and self.album_type == other.album_type \
            and self.spotify_id == other.spotify_id \
            and self.release_date_precision == self.release_date_precision \
            and self.release_date == self.release_date
    
    def __repr__(self):
        return str(self.__dict__)

SINGLE_ALBUM_TYPE = "single"
ALBUM_ALBUM_TYPE = "album"


import logging
import requests
from lastipy.recommendations.recommended_track import RecommendedTrack
from lastipy.lastfm.parse_lastfm_tracks import parse_track_name, parse_artist

URL = 'http://ws.audioscrobbler.com/2.0/?method=track.getsimilar'


def fetch_recommendations(api_key, track, limit):
    """Fetches tracks similar to the given track.
    :param limit: the maximum number of items to fetch"""

    logging.info("Fetching up to " + str(limit) +
                 " recommendations based on " + str(track) + " in Last.fm")
    json_response = _send_request(_build_json_payload(api_key, track, limit))
    if 'similartracks' in json_response:
        json_tracks = json_response['similartracks']['track']
        similar_tracks = [RecommendedTrack(track_name=parse_track_name(json_track), artist=parse_artist(json_track))
                          for json_track in json_tracks]
        logging.info("Fetched " + str(len(similar_tracks)) + " track recommendations")
        logging.debug("Fetched tracks: " + str(similar_tracks))
        return similar_tracks
    elif 'errors' in json_response:
        raise Exception(
            "Error occurred while fetching similar tracks " + json_response['errors'])


def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status()


def _build_json_payload(api_key, track, limit):
    payload = {
        'track': track.track_name,
        'artist': track.artist,
        'format': 'json',
        'api_key': api_key,
        'limit': limit
    }
    return payload



import unittest
from unittest.mock import patch
from unittest.mock import Mock
from requests import HTTPError
from lastipy.track import Track
from lastipy.lastfm.library.paginated_endpoint import fetch_paginated_response


class RecentArtistsTest(unittest.TestCase):

    dummy_url = 'www.example.org'
    dummy_user = 'dummyUser'
    dummy_api_key = '123456789'

    @patch('requests.get')
    def test_fetch_single_page(self, mock_requests_get):
        mock_response = Mock()
        mock_response.ok = True
        json_object_page = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value1',
                        'key2': {
                            'key3': 'Value2'
                        }
                    },
                    {
                        'key1': 'Value3',
                        'key2': {
                            'key3': 'Value4'
                        }
                    }
                ],
                '@attr': {
                    'totalPages': '1'
                }
            }
        }
        mock_response.json.return_value = json_object_page
        mock_requests_get.side_effect = [mock_response]

        fetched_objects = fetch_paginated_response(
            self.dummy_url, self.dummy_user, self.dummy_api_key, 'arrayKey')

        self.assertCountEqual(fetched_objects, [json_object_page])

    @patch('requests.get')
    def test_fetch_multiple_pages(self, mock_requests_get):
        json_object_page_1 = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value1',
                        'key2': 'Value2'
                    },
                    {
                        'key1': 'Value3',
                        'key2': 'Value4'
                    }
                ],
                '@attr': {
                    'totalPages': '3'
                }
            }
        }
        json_object_page_2 = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value5',
                        'key2': 'Value6'
                    },
                    {
                        'key1': 'Value7',
                        'key2': 'Value8'
                    }
                ],
                '@attr': {
                    'totalPages': '3'
                }
            }
        }
        json_object_page_3 = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value9',
                        'key2': 'Value10'
                    },
                    {
                        'key1': 'Value11',
                        'key2': 'Value12'
                    }
                ],
                '@attr': {
                    'totalPages': '3'
                }
            }
        }

        mock_responses = [Mock(), Mock(), Mock()]
        mock_responses[0].ok = True
        mock_responses[0].json.return_value = json_object_page_1
        mock_responses[1].ok = True
        mock_responses[1].json.return_value = json_object_page_2
        mock_responses[2].ok = True
        mock_responses[2].json.return_value = json_object_page_3
        mock_requests_get.side_effect = mock_responses

        fetched_objects = fetch_paginated_response(
            self.dummy_url, self.dummy_user, self.dummy_api_key, 'arrayKey')

        self.assertCountEqual(
            fetched_objects, [json_object_page_1, json_object_page_2, json_object_page_3])

    @patch('requests.get')
    def test_fetch_with_success_after_retries(self, mock_requests_get):
        json_object_page_1 = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value1',
                        'key2': 'Value2'
                    },
                    {
                        'key1': 'Value3',
                        'key2': 'Value4'
                    }
                ],
                '@attr': {
                    'totalPages': '2'
                }
            }
        }

        json_object_page_2 = {
            'arrayKey': {
                'array': [
                    {
                        'key1': 'Value5',
                        'key2': 'Value6'
                    },
                    {
                        'key1': 'Value7',
                        'key2': 'Value8'
                    }
                ],
                '@attr': {
                    'totalPages': '2'
                }
            }
        }

        mock_responses = [Mock(), Mock(), Mock()]
        mock_responses[0].ok = True
        mock_responses[0].json.return_value = json_object_page_1
        mock_responses[1].ok = False
        mock_responses[1].raise_for_status.side_effect = HTTPError(
            Mock(status=500), 'Error')
        mock_responses[2].ok = True
        mock_responses[2].json.return_value = json_object_page_2
        mock_requests_get.side_effect = mock_responses

        fetched_objects = fetch_paginated_response(
            self.dummy_url, self.dummy_user, self.dummy_api_key, 'arrayKey')

        self.assertCountEqual(
            fetched_objects, [json_object_page_1, json_object_page_2])

    @patch('requests.get')
    def test_fetch_fails_after_retries(self, mock_requests_get):
        mock_responses = []
        for _ in range(11):
            mock_response = Mock()
            mock_response.ok = False
            mock_response.raise_for_status.side_effect = HTTPError(
                Mock(status=500), 'Error')
            mock_responses.append(mock_response)

        # Add another mock response, but the code will exit after the retry limit is reached and this won't actually get fetched
        mock_response = Mock()
        mock_response.ok = True
        mock_response.json.return_value = {'test'}
        mock_responses.append(mock_response)

        mock_requests_get.side_effect = mock_responses

        fetched_objects = fetch_paginated_response(
            self.dummy_url, self.dummy_user, self.dummy_api_key, 'arrayKey')

        self.assertCountEqual(
            fetched_objects, [])

    @patch('requests.get')
    def test_extra_request_params(self, mock_requests_get):
        mock_requests_get.ok = True
        mock_requests_get.json.return_value = []

        fetch_paginated_response(self.dummy_url, self.dummy_user, self.dummy_api_key, 'arrayKey', [
            {'key': 'key1', 'value': 'value1'}, {'key': 'key2', 'value': 'value2'}])

        expected_request = {
            'user': self.dummy_user,
            'format': 'json',
            'api_key': self.dummy_api_key,
            'limit': 200,
            'page': 1,
            'key1': 'value1',
            'key2': 'value2'
        }

        mock_requests_get.assert_called_with(
            self.dummy_url, params=expected_request)



import logging
import requests
from lastipy.lastfm.library.top_track import TopTrack
from requests import RequestException

URL = 'http://ws.audioscrobbler.com/2.0/?method=track.getInfo'
MAX_RETRIES = 10


# TODO rename - track_info kinda vague
def fetch_playcount(track, user, api_key):
    retries = 0
    while retries <= MAX_RETRIES:
        try:
            json_response = _send_request(_build_payload(track, user, api_key))
            playcount = int(json_response['track']['userplaycount'])
            logging.debug(user + " has played " + str(track) +
                          " " + str(playcount) + " times")
            return playcount
        except RequestException:
            if retries < MAX_RETRIES:
                logging.warning(
                    "Failed to fetch playcount for track " + str(track) + ". Retrying...")
                retries += 1
            else:
                logging.warning("Failed to fetch playcount after " +
                                str(retries) + " retries. Returning a playcount of 0.")
                return 0


def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status()


def _build_payload(track, user, api_key):
    payload = {
        'username': user,
        'api_key': api_key,
        'format': 'json',
        'track': track.track_name,
        'artist': track.artist,
        'autocorrect': 1
    }
    return payload



import unittest
from unittest.mock import patch
from lastipy.lastfm.library.track_info import fetch_playcount
from lastipy.track import Track
from unittest.mock import Mock
from requests import HTTPError


class FetchPlaycountTest(unittest.TestCase):

    dummy_track = Track(track_name="Dummy Track", artist="Dummy Artist")

    @patch('requests.get')
    def test_successful_fetch(self, mock_requests_get):
        json_response = {
            'track': {
                'userplaycount': '5'
            }
        }

        mock_response = Mock()
        mock_response.ok = True
        mock_response.json.return_value = json_response
        mock_requests_get.side_effect = [mock_response]

        fetched_playcount = fetch_playcount(track=self.dummy_track, user='dummyUser',
                                            api_key='dummyApiKey')

        self.assertEqual(fetched_playcount, 5)

    @patch('requests.get')
    def test_successful_fetch_after_retries(self, mock_requests_get):
        json_response = {
            'track': {
                'userplaycount': '7'
            }
        }

        mock_responses = [Mock(), Mock(), Mock()]
        mock_responses[0].ok = False
        mock_responses[0].raise_for_status.side_effect = HTTPError(
            Mock(status=500), "Error")
        mock_responses[1] = mock_responses[0]
        mock_responses[2].ok = True
        mock_responses[2].json.return_value = json_response
        mock_requests_get.side_effect = mock_responses

        fetched_playcount = fetch_playcount(
            track=self.dummy_track, user='dummyUser', api_key='dummyApiKey')

        self.assertEqual(fetched_playcount, 7)

    @patch('requests.get')
    def test_failure_after_retries(self, mock_requests_get):
        mock_responses = []
        for _ in range(11):
            mock_response = Mock()
            mock_response.ok = False
            mock_response.raise_for_status.side_effect = HTTPError(
                Mock(status=500), "Error")
            mock_responses.append(mock_response)

        mock_requests_get.side_effect = mock_responses

        fetched_playcount = fetch_playcount(
            track=self.dummy_track, user="dummyUser", api_key="dummyApiKey")

        self.assertEqual(0, fetched_playcount)



from configparser import ConfigParser


class ApiKeysParser:
    
    def __init__(self, file):
        self._extract_keys(file)        

    def _extract_keys(self, file):
        config_parser = ConfigParser()
        config_parser.read(file.name)
        self.lastfm_api_key = config_parser['LastFM']['API']
        spotify_section = 'Spotify'
        self.spotify_client_id_key = config_parser[spotify_section]['CLIENT_ID']
        self.spotify_client_secret_key = config_parser[spotify_section]['CLIENT_SECRET']



import spotipy
from lastipy.spotify.parse_spotify_tracks import parse_tracks
from lastipy.spotify.library import get_saved_tracks
from lastipy.spotify.playlist import get_tracks_in_playlists
from datetime import datetime
import logging
from lastipy.track import Track
from lastipy.spotify import album


# TODO test
def fetch_new_tracks(spotify, album_types=[album.SINGLE_ALBUM_TYPE, album.ALBUM_ALBUM_TYPE], ignore_remixes=False, ignore_songs_in_library=True, as_of_date=datetime.today().date()):
    """Fetches new tracks (as of the given date) released by the current Spotify user's followed artists"""

    logging.info("Fetching new tracks for " + spotify.current_user()
                 ['id'] + " as of " + str(as_of_date))

    new_albums = fetch_new_albums(spotify, album_types, as_of_date)

    all_tracks = []
    for album in new_albums:
        all_tracks += _fetch_album_tracks(spotify, album)

    new_tracks = parse_tracks(all_tracks)
    new_tracks = _remove_duplicates(new_tracks)

    if ignore_remixes:
        logging.info("Filtering out those pesky remixes...")
        new_tracks = [
            track for track in new_tracks if "remix" not in track.track_name.lower()]

    if ignore_songs_in_library:
        logging.info(
            "Filtering out tracks that are already in the user's saved tracks and playlists...")
        saved_tracks = get_saved_tracks(spotify)
        tracks_in_playlists = get_tracks_in_playlists(spotify)
        new_tracks = [new_track for new_track in new_tracks
                      if not any(Track.are_equivalent(new_track, saved_track) for saved_track in saved_tracks)]
        new_tracks = [new_track for new_track in new_tracks
                      if not any(Track.are_equivalent(new_track, playlist_track) for playlist_track in tracks_in_playlists)]

    logging.info("Fetched " + str(len(new_tracks)) +
                 " new tracks " + str(new_tracks))
    return new_tracks

# TODO test


def fetch_new_albums(spotify, album_types=[album.SINGLE_ALBUM_TYPE, album.ALBUM_ALBUM_TYPE], as_of_date=datetime.today().date()):
    """Fetches new albums (as of the given date) released by the given Spotify user's followed artists"""

    followed_artist_ids = _fetch_followed_artists(spotify)

    logging.info("Fetching new albums for " + spotify.current_user()
                 ['id'] + " as of " + str(as_of_date))

    all_albums = []
    for artist_id in followed_artist_ids:
        artist_albums = _fetch_artist_albums(spotify, album_types, artist_id)
        all_albums += artist_albums

    new_albums = _filter_new_albums(all_albums, as_of_date)

    logging.info("Fetched " + str(len(new_albums)) +
                 " new albums " + str(new_albums))
    return new_albums


def _fetch_followed_artists(spotify):
    followed_artists = []

    curr_response = spotify.current_user_followed_artists(limit=50)

    while len(curr_response['artists']['items']) > 0:
        curr_response = spotify.current_user_followed_artists(
            limit=50, after=curr_response['artists']['items'][len(curr_response) - 1]['id'])
        followed_artists += curr_response['artists']['items']

    # The above Spotipy function doesn't really seem to function properly and results in duplicates,
    # so we remove them here by converting the list to just the IDs (not doing so results in
    # an unhashable error), then converting to a set and back to a list
    followed_artists = [artist['id'] for artist in followed_artists]
    followed_artist_ids = list(set(followed_artists))
    return followed_artist_ids


def _filter_new_albums(all_albums, as_of_date):
    new_albums = []
    for album in all_albums:
        if album.release_date_precision == 'day':
            if datetime.strptime(album.release_date, "%Y-%m-%d").date() == as_of_date:
                new_albums.append(album)
        else:
            logging.warn(
                "Album release date precision is not 'day' so ignoring (album: " + str(album) + ")")
    return new_albums


def _fetch_artist_albums(spotify, album_types, artist_id):
    # TODO definitely need to extract something here...
    albums = []
    if album.ALBUM_ALBUM_TYPE in album_types:
        curr_response = spotify.artist_albums(
            artist_id, album_type='album', limit=50)
        albums = _convert_albums(curr_response, 'album')
        while len(curr_response['items']) > 0:
            curr_response = spotify.artist_albums(
                artist_id, album_type='album', limit=50, offset=len(albums))
            albums += _convert_albums(curr_response, 'album')

    singles = []
    if album.SINGLE_ALBUM_TYPE in album_types:
        curr_response = spotify.artist_albums(
            artist_id, album_type='single', limit=50)
        singles = _convert_albums(curr_response, 'single')
        while len(curr_response['items']) > 0:
            curr_response = spotify.artist_albums(
                artist_id, album_type='single', limit=50, offset=len(singles))
            singles += _convert_albums(curr_response, 'single')

    return albums + singles


def _convert_albums(json_album_response, album_type):
    return [album.SpotifyAlbum(album_type=album_type, spotify_id=item['id'], release_date_precision=item['release_date_precision'], release_date=item['release_date']) for item in json_album_response['items']]


def _fetch_album_tracks(spotify, album):
    curr_response = spotify.album_tracks(album.spotify_id, limit=50)
    album_tracks = curr_response['items']
    while len(curr_response['items']) > 0:
        curr_response = spotify.album_tracks(
            album.spotify_id, limit=50, offset=len(album_tracks))
        album_tracks += curr_response['items']
    return album_tracks


def _remove_duplicates(tracks):
    """Removes duplicate tracks (ie: tracks that have identical name and artist; see Track.are_equivalent)"""
    logging.debug("New tracks before removing duplicates: " + str(tracks))
    tracks_without_duplicates = []
    # We remove duplicates with a list comprehension rather than the traditional hack of using a set, since that
    # requires the object to be hashable; plus we only want to compare the track name/artist of each track, not
    # any of the other fields (eg: Spotify ID) which might in fact differ
    [tracks_without_duplicates.append(track_x)
        for track_x in tracks
        if not any(Track.are_equivalent(track_x, track_y)
                   for track_y in tracks_without_duplicates)]
    logging.debug("New tracks after removing duplicates: " +
                  str(tracks_without_duplicates))
    return tracks_without_duplicates



import unittest
from lastipy.spotify.library import add_tracks_to_library
from spotipy import Spotify
from lastipy.track import Track


class AddTracksToLibraryTest(unittest.TestCase):

    def test_adding_less_than_max_request(self):
        spotify = Spotify()
        spotify.current_user = unittest.mock.MagicMock({'id': 'testUser'})
        spotify.current_user_saved_tracks_add = unittest.mock.MagicMock()
        dummy_tracks = [
            self._build_dummy_track('123'),
            self._build_dummy_track('456'),
            self._build_dummy_track('789')
        ]
        add_tracks_to_library(spotify, dummy_tracks)
        spotify.current_user_saved_tracks_add.assert_called_with(
            ['123', '456', '789'])

    def test_adding_more_than_max_request(self):
        spotify = Spotify()
        spotify.current_user = unittest.mock.MagicMock({'id': 'testUser'})
        spotify.current_user_saved_tracks_add = unittest.mock.MagicMock()

        dummy_tracks = []
        for _ in range(150):
            dummy_tracks.append(self._build_dummy_track('123'))

        expected_chunks = []
        for _ in range(3):
            chunk = []
            for _ in range(50):
                chunk.append('123')
            expected_chunks.append(chunk)

        add_tracks_to_library(spotify, dummy_tracks)

        for i in range(3):
            spotify.current_user_saved_tracks_add.assert_called_with(
                expected_chunks[i])

    def _build_dummy_track(self, spotify_id):
        return Track(spotify_id=spotify_id, track_name='dummy_track_name', artist='dummy_artist')



from setuptools import setup, find_packages

setup(name='lastipy',
      version='1.1.4',
      description='Python library that combines Last.fm and Spotify',
      url='http://github.com/evanjamesjackson/lastipy',
      author='Evan Jackson',
      author_email='evanjamesjackson@gmail.com',
      packages=find_packages(),
      entry_points={'console_scripts': [
          'recommendations_playlist = scripts.recommendations_playlist:build_recommendations_playlist',
          'save_new_releases = scripts.save_new_releases:save_new_releases',
          'organize_favorites = scripts.organize_favorites:organize_favorites'
      ]},
      install_requires=['numpy', 'requests', 'spotipy', 'pytest', 'iso8601', 'python-dateutil'])



import logging

from lastipy.lastfm.library.top_tracks import fetch_top_tracks 
from lastipy.lastfm import lastfm_recommendations
from lastipy.spotify import spotify_recommendations
from lastipy.lastfm.library import period
from lastipy.track import Track
from lastipy.lastfm.library.recent_tracks import fetch_recent_tracks
from lastipy.recommendations.rating_calculator import calculate_ratings


def generate_recommendations(
          user,
          lastfm_api_key,
          spotify,
          recommendation_services='Last.fm',
          recommendation_period=period.OVERALL,
          max_recommendations_per_top_track=100,
          blacklisted_artists=[],
          prefer_unheard_artists=True):
    """Fetches recommendations for the given user by fetching their top tracks, then getting tracks similar
    to them, and filtering out the user's recent tracks and blacklisted artists"""

    logging.info("Fetching recommendations for " + user)

    top_tracks = fetch_top_tracks(user=user, api_key=lastfm_api_key, a_period=recommendation_period)

    top_tracks_to_recommendations = {}
    recommendations = []
    for top_track in top_tracks:
        try:
            recommendations_for_current_track = _fetch_recommendations(recommendation_services, lastfm_api_key, spotify, top_track, max_recommendations_per_top_track) 
            if recommendations_for_current_track:
                recommendations = recommendations + recommendations_for_current_track
                top_tracks_to_recommendations[top_track] = recommendations_for_current_track
        except Exception as e:
            logging.error(f"An error occurred fetching recommendations for track " + str(top_track) + ": " + str(e))

    recommendations = calculate_ratings(user=user,
                                api_key=lastfm_api_key,
                                prefer_unheard_artists=prefer_unheard_artists,
                                top_tracks_to_recommendations=top_tracks_to_recommendations)

    logging.debug(f"Before filtering, fetched " + str(len(recommendations))
                    + " recommendations: " + str(recommendations))

    recommendations = _filter_out_recent_tracks(user, lastfm_api_key, recommendations)

    recommendations = _filter_out_blacklisted_artists(blacklisted_artists, recommendations)

    logging.info("Aftering filtering, fetched " + str(len(recommendations)) + " recommendations")
    logging.debug("Fetched tracks: " + str(recommendations))
    return recommendations

def _filter_out_blacklisted_artists(blacklisted_artists, recommendations):
    logging.info("Filtering out blacklisted artists (" + str(blacklisted_artists) + ")")
    recommendations = [recommendation for recommendation in recommendations
                        if not any(recommendation.artist.lower() == blacklisted_artist.lower()
                                    for blacklisted_artist in blacklisted_artists)]
    return recommendations

def _filter_out_recent_tracks(user, api_key, recommendations):
    recent_tracks = fetch_recent_tracks(user, api_key)
    logging.info("Filtering out recent tracks from recommendations")
    recommendations = [recommendation for recommendation in recommendations
                        if not any(Track.are_equivalent(recommendation, recent_track)
                                    for recent_track in recent_tracks)]
    return recommendations

def _fetch_recommendations(recommendation_services, lastfm_api_key, spotify, track, limit): 
    recommendations = []
    if 'Last.fm' in recommendation_services: 
        recommendations += lastfm_recommendations.fetch_recommendations(api_key=lastfm_api_key, limit=limit, track=track)
    if 'Spotify' in recommendation_services:
        recommendations += spotify_recommendations.fetch_recommendations(spotify=spotify, track=track, limit=limit)
    return recommendations



import logging
from lastipy.lastfm.parse_lastfm_tracks import parse_artist, parse_track_name
from lastipy.track import Track
from lastipy.lastfm.library.paginated_endpoint import fetch_paginated_response

URL = 'http://ws.audioscrobbler.com/2.0/?method=user.getrecenttracks'


def fetch_recent_tracks(user, api_key):
    """Fetches recent tracks for the given user"""

    logging.info("Fetching recent tracks for " + user)

    paginated_json_responses = fetch_paginated_response(
        URL, user, api_key, 'recenttracks')
    recent_tracks = []
    for json_response in paginated_json_responses:
        for track in json_response['recenttracks']['track']:
            recent_tracks.append(Track(parse_track_name(
                track), parse_artist(track)))

    logging.info("Fetched " + str(len(recent_tracks)) + " recent tracks")
    logging.debug("Fetched tracks: " + str(recent_tracks))
    return recent_tracks
